package java.lang;

import sun.nio.cs.HistoricallyNamedCharset;
import java.nio.charset.CharsetEncoder;
import sun.nio.cs.ArrayEncoder;
import java.nio.charset.CoderResult;
import java.nio.charset.CharsetDecoder;
import java.nio.charset.CharacterCodingException;
import java.nio.CharBuffer;
import java.nio.ByteBuffer;
import sun.nio.cs.ArrayDecoder;
import java.nio.charset.CodingErrorAction;
import java.io.UnsupportedEncodingException;
import java.nio.charset.IllegalCharsetNameException;
import sun.misc.MessageUtils;
import java.nio.charset.UnsupportedCharsetException;
import java.util.Arrays;
import java.nio.charset.Charset;
import java.lang.ref.SoftReference;

class StringCoding
{
    private static final ThreadLocal<SoftReference<StringCoding.StringDecoder>> decoder;
    private static final ThreadLocal<SoftReference<StringCoding.StringEncoder>> encoder;
    private static boolean warnUnsupportedCharset;
    
    private StringCoding() {
    }
    
    private static <T> T deref(ThreadLocal<SoftReference<T>> threadLocal) {
        SoftReference<Object> softReference = (SoftReference)threadLocal.get();
        if (softReference == null) {
            return null;
        }
        return softReference.get();
    }
    
    private static <T> void set(ThreadLocal<SoftReference<T>> threadLocal, T referent) {
        threadLocal.set(new SoftReference(referent));
    }
    
    private static byte[] safeTrim(byte[] original, int newLength, Charset charset, boolean b) {
        if (newLength == original.length && (b || System.getSecurityManager() == null)) {
            return original;
        }
        return Arrays.copyOf(original, newLength);
    }
    
    private static char[] safeTrim(char[] original, int newLength, Charset charset, boolean b) {
        if (newLength == original.length && (b || System.getSecurityManager() == null)) {
            return original;
        }
        return Arrays.copyOf(original, newLength);
    }
    
    private static int scale(int n, float n2) {
        return (int)((double)n * (double)n2);
    }
    
    private static Charset lookupCharset(String s) {
        if (Charset.isSupported(s)) {
            try {
                return Charset.forName(s);
            }
            catch (UnsupportedCharsetException ex) {
                throw new Error(ex);
            }
        }
        else {
            return null;
        }
    }
    
    private static void warnUnsupportedCharset(String s) {
        if (StringCoding.warnUnsupportedCharset) {
            MessageUtils.err("WARNING: Default charset " + s + " not supported, using ISO-8859-1 instead");
            StringCoding.warnUnsupportedCharset = false;
        }
    }
    
    static char[] decode(String s, byte[] array, int n, int n2) throws UnsupportedEncodingException {
        StringCoding.StringDecoder stringDecoder = (StringCoding.StringDecoder)deref(StringCoding.decoder);
        s = ((s == null) ? "ISO-8859-1" : s);
        if (stringDecoder == null || (!s.equals(stringDecoder.requestedCharsetName()) && !s.equals(stringDecoder.charsetName()))) {
            stringDecoder = null;
            try {
                Charset lookupCharset = lookupCharset(s);
                if (lookupCharset != null) {
                    stringDecoder = new StringCoding.StringDecoder(lookupCharset, s);
                }
            }
            catch (IllegalCharsetNameException ex) {}
            if (stringDecoder == null) {
                throw new UnsupportedEncodingException(s);
            }
            set(StringCoding.decoder, stringDecoder);
        }
        return stringDecoder.decode(array, n, n2);
    }
    
    static char[] decode(Charset charset, byte[] copyOfRange, int n, int length) {
        CharsetDecoder decoder = charset.newDecoder();
        char[] array = new char[scale(length, decoder.maxCharsPerByte())];
        if (length == 0) {
            return array;
        }
        boolean n2 = false;
        if (System.getSecurityManager() != null && (n2 = (charset.getClass().getClassLoader0() == null)) == 0) {
            copyOfRange = Arrays.copyOfRange(copyOfRange, n, n + length);
            n = 0;
        }
        decoder.onMalformedInput(CodingErrorAction.REPLACE).onUnmappableCharacter(CodingErrorAction.REPLACE).reset();
        if ((decoder instanceof ArrayDecoder)) {
            return safeTrim(array, ((ArrayDecoder)decoder).decode(copyOfRange, n, length, array), charset, (boolean)n2);
        }
        ByteBuffer wrap = ByteBuffer.wrap(copyOfRange, n, length);
        CharBuffer wrap2 = CharBuffer.wrap(array);
        try {
            CoderResult decode = decoder.decode(wrap, wrap2, true);
            if (!decode.isUnderflow()) {
                decode.throwException();
            }
            CoderResult flush = decoder.flush(wrap2);
            if (!flush.isUnderflow()) {
                flush.throwException();
            }
        }
        catch (CharacterCodingException ex) {
            throw new Error(ex);
        }
        return safeTrim(array, wrap2.position(), charset, (boolean)n2);
    }
    
    static char[] decode(byte[] array, int n, int n2) {
        String name = Charset.defaultCharset().name();
        try {
            return decode(name, array, n, n2);
        }
        catch (UnsupportedEncodingException ex) {
            warnUnsupportedCharset(name);
            try {
                return decode("ISO-8859-1", array, n, n2);
            }
            catch (UnsupportedEncodingException ex2) {
                MessageUtils.err("ISO-8859-1 charset not available: " + ex2.toString());
                System.exit(1);
                return null;
            }
        }
    }
    
    static byte[] encode(String s, char[] array, int n, int n2) throws UnsupportedEncodingException {
        StringCoding.StringEncoder stringEncoder = (StringCoding.StringEncoder)deref(StringCoding.encoder);
        s = ((s == null) ? "ISO-8859-1" : s);
        if (stringEncoder == null || (!s.equals(stringEncoder.requestedCharsetName()) && !s.equals(stringEncoder.charsetName()))) {
            stringEncoder = null;
            try {
                Charset lookupCharset = lookupCharset(s);
                if (lookupCharset != null) {
                    stringEncoder = new StringCoding.StringEncoder(lookupCharset, s);
                }
            }
            catch (IllegalCharsetNameException ex) {}
            if (stringEncoder == null) {
                throw new UnsupportedEncodingException(s);
            }
            set(StringCoding.encoder, stringEncoder);
        }
        return stringEncoder.encode(array, n, n2);
    }
    
    static byte[] encode(Charset charset, char[] copyOfRange, int n, int length) {
        CharsetEncoder encoder = charset.newEncoder();
        byte[] array = new byte[scale(length, encoder.maxBytesPerChar())];
        if (length == 0) {
            return array;
        }
        boolean n2 = false;
        if (System.getSecurityManager() != null && (n2 = (charset.getClass().getClassLoader0() == null)) == 0) {
            copyOfRange = Arrays.copyOfRange(copyOfRange, n, n + length);
            n = 0;
        }
        encoder.onMalformedInput(CodingErrorAction.REPLACE).onUnmappableCharacter(CodingErrorAction.REPLACE).reset();
        if ((encoder instanceof ArrayEncoder)) {
            return safeTrim(array, ((ArrayEncoder)encoder).encode(copyOfRange, n, length, array), charset, (boolean)n2);
        }
        ByteBuffer wrap = ByteBuffer.wrap(array);
        CharBuffer wrap2 = CharBuffer.wrap(copyOfRange, n, length);
        try {
            CoderResult encode = encoder.encode(wrap2, wrap, true);
            if (!encode.isUnderflow()) {
                encode.throwException();
            }
            CoderResult flush = encoder.flush(wrap);
            if (!flush.isUnderflow()) {
                flush.throwException();
            }
        }
        catch (CharacterCodingException ex) {
            throw new Error(ex);
        }
        return safeTrim(array, wrap.position(), charset, (boolean)n2);
    }
    
    static byte[] encode(char[] array, int n, int n2) {
        String name = Charset.defaultCharset().name();
        try {
            return encode(name, array, n, n2);
        }
        catch (UnsupportedEncodingException ex) {
            warnUnsupportedCharset(name);
            try {
                return encode("ISO-8859-1", array, n, n2);
            }
            catch (UnsupportedEncodingException ex2) {
                MessageUtils.err("ISO-8859-1 charset not available: " + ex2.toString());
                System.exit(1);
                return null;
            }
        }
    }
    
    static int access$000(int n, float n2) {
        return scale(n, n2);
    }
    
    static char[] access$100(char[] array, int n, Charset charset, boolean b) {
        return safeTrim(array, n, charset, b);
    }
    
    static byte[] access$300(byte[] array, int n, Charset charset, boolean b) {
        return safeTrim(array, n, charset, b);
    }
    
    static {
        decoder = new ThreadLocal();
        encoder = new ThreadLocal();
        StringCoding.warnUnsupportedCharset = true;
    }
    
    private static class StringDecoder
    {
        private final String requestedCharsetName;
        private final Charset cs;
        private final CharsetDecoder cd;
        private final boolean isTrusted;
        
        private StringDecoder(Charset cs, String requestedCharsetName) {
            this.requestedCharsetName = requestedCharsetName;
            this.cs = cs;
            this.cd = cs.newDecoder().onMalformedInput(CodingErrorAction.REPLACE).onUnmappableCharacter(CodingErrorAction.REPLACE);
            this.isTrusted = (cs.getClass().getClassLoader0() == null);
        }
        
        String charsetName() {
            if ((this.cs instanceof HistoricallyNamedCharset)) {
                return ((HistoricallyNamedCharset)this.cs).historicalName();
            }
            return this.cs.name();
        }
        
        final String requestedCharsetName() {
            return this.requestedCharsetName;
        }
        
        char[] decode(byte[] array, int offset, int length) {
            char[] array2 = new char[StringCoding.access$000(length, this.cd.maxCharsPerByte())];
            if (length == 0) {
                return array2;
            }
            if ((this.cd instanceof ArrayDecoder)) {
                return StringCoding.access$100(array2, ((ArrayDecoder)this.cd).decode(array, offset, length, array2), this.cs, this.isTrusted);
            }
            this.cd.reset();
            ByteBuffer wrap = ByteBuffer.wrap(array, offset, length);
            CharBuffer wrap2 = CharBuffer.wrap(array2);
            try {
                CoderResult decode = this.cd.decode(wrap, wrap2, true);
                if (!decode.isUnderflow()) {
                    decode.throwException();
                }
                CoderResult flush = this.cd.flush(wrap2);
                if (!flush.isUnderflow()) {
                    flush.throwException();
                }
            }
            catch (CharacterCodingException ex) {
                throw new Error(ex);
            }
            return StringCoding.access$100(array2, wrap2.position(), this.cs, this.isTrusted);
        }
    }
    
    private static class StringEncoder
    {
        private Charset cs;
        private CharsetEncoder ce;
        private final String requestedCharsetName;
        private final boolean isTrusted;
        
        private StringEncoder(Charset cs, String requestedCharsetName) {
            this.requestedCharsetName = requestedCharsetName;
            this.cs = cs;
            this.ce = cs.newEncoder().onMalformedInput(CodingErrorAction.REPLACE).onUnmappableCharacter(CodingErrorAction.REPLACE);
            this.isTrusted = (cs.getClass().getClassLoader0() == null);
        }
        
        String charsetName() {
            if ((this.cs instanceof HistoricallyNamedCharset)) {
                return ((HistoricallyNamedCharset)this.cs).historicalName();
            }
            return this.cs.name();
        }
        
        final String requestedCharsetName() {
            return this.requestedCharsetName;
        }
        
        byte[] encode(char[] array, int offset, int length) {
            byte[] array2 = new byte[StringCoding.access$000(length, this.ce.maxBytesPerChar())];
            if (length == 0) {
                return array2;
            }
            if ((this.ce instanceof ArrayEncoder)) {
                return StringCoding.access$300(array2, ((ArrayEncoder)this.ce).encode(array, offset, length, array2), this.cs, this.isTrusted);
            }
            this.ce.reset();
            ByteBuffer wrap = ByteBuffer.wrap(array2);
            CharBuffer wrap2 = CharBuffer.wrap(array, offset, length);
            try {
                CoderResult encode = this.ce.encode(wrap2, wrap, true);
                if (!encode.isUnderflow()) {
                    encode.throwException();
                }
                CoderResult flush = this.ce.flush(wrap);
                if (!flush.isUnderflow()) {
                    flush.throwException();
                }
            }
            catch (CharacterCodingException ex) {
                throw new Error(ex);
            }
            return StringCoding.access$300(array2, wrap.position(), this.cs, this.isTrusted);
        }
    }
}
